from subprocess import call
from cmd import Cmd
from os import isatty
from select import poll, POLLIN
import select
import errno
import sys
import time
import os, os.path
import atexit
import time
import re
import threading
from Kitsune import Kitsune
import numpy as np
import signal
import statistics

tcpdumpProc = False

def signal_handler(sig, frame):
    global tcpdumpProc
    if(tcpdumpProc):
        cmd = "sudo pkill tcpdump"
        call(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, shell=True)
    sys.exit(0)

def dirHelper(pathIn):
    if(os.path.isdir(pathIn)):
       for a_file in os.listdir(pathIn):
           file_path = os.path.join(pathIn, a_file)
           try:
               if(os.path.isfile(file_path)):
                   os.unlink(file_path)
           except Exception as e:
               print(e)

    else:
       print("creating directory...")
       os.mkdir(pathIn)

def runIDS(cutoff=0.75, trainSize=40000, interval=10, maxAE=10):
      print("Running IDS in real-time")
      # Create dir if doesn't exists, clear it if it does
      curDir = os.getcwd()
      dataPath = curDir + "/live_data"
      anomalyPath = curDir + "/anomaly_log"
      print("Cleaning any previous run files...")
      dirHelper(dataPath)
      dirHelper(anomalyPath)

      # Switch to directory and capture training data
      os.chdir(dataPath)
      print("Capturing traing data. This may take a while...")
      trainString = "sudo tcpdump -i wlan0 -w training.pcap -c " + str(trainSize)
      p = subprocess.Popen(trainString, stdout=subprocess.PIPE, shell=True)
      p.wait()
      if(p.returncode == 0):
          print("Captured training data successfully!")
          time.sleep(30)       # Testing to see if i need to allow time for pcap to process
      else:
          print("Capture of training data failed...aborting")
          return False

      # After successful training capture in specified intervals, defaults to 10 sec
      pktCap(interval, dataPath)

      # Start Kitsune
      packet_limit = np.Inf #the number of packets to process

      # KitNET params:
      maxAE = 10 #maximum size for any autoencoder in the ensemble layer
      FMgrace = 1000 #the number of instances taken to learn the feature mapping (the ensemble's architecture)
      ADgrace = trainSize - FMgrace #the number of instances used to train the anomaly detector (ensemble itself)

      # Build Kitsune
      print("Path passed to Kitsune: " + dataPath)
      K = Kitsune(dataPath, packet_limit, maxAE, FMgrace, ADgrace, cutoff)

      print("Running Kitsune:")
      RMSEs = []
      std = []
      slope = []
      pktCount = 0
      stdCount = 0
      writePath = anomalyPath + "/log.txt"
      file = open(writePath, "w")
      # Here we process (train/execute) each individual packet.
      # In this way, each observation is discarded after performing process() method.
      while True:
          rmse = K.proc_next_packet()
          #if rmse == -1:
              #print("waiting for additional files")

          if(rmse is not None and rmse != -1):
              # Use moving stdev to filter data and reduce false positives
              pktCount+=1
              RMSEs.append(rmse)
              if(pktCount >= 20):
                  stdCount += 1
                  std.append(statistics.stdev(RMSEs[(pktCount - 20):(pktCount - 1)]))
                  if(stdCount >= 5):
                      slope.append((std[stdCount - 5] - std[stdCount - 1])/5)
                      if(slope[stdCount - 5] >= cutoff):
                          # Anomaly triggered, write MACs to file
                          anomalyToFile(K.triggerMACs, file)

def anomalyToFile(macs, file):
    if(macs is not None):
        file.write(str(macs[0]) + " " + str(macs[1]) + "\n")
    else:
        print("triggered and confused...")

def pktCap(interval, dataPath):
  os.chdir(dataPath)
  captureThread = threading.Thread(target=capLoop, args=[interval])
  captureThread.start()

def capLoop(interval):
  global tcpdumpProc
  tcpdumpProc = True
  cmd = "sudo tcpdump -i wlan0 -w trace-%m-%d-%H-%M-%S.pcap -G " + str(interval)
  call(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, shell=True)


# Run
signal.signal(signal.SIGINT, signal_handler)    # catches control c and kills tcpdump
#runIDS(10, 2000, 15, 15)
