from argparse import ArgumentParser
from os.path import exists, join
from os import mkdir
from threading import Thread
from time import time
from scapy.all import sniff
from pickle import dump

DEVICES = '/home/pi/MQP/degradation/devices'

_interval_bounds = []
_packet_timestamps = []
_packets = []

_start_time = time()

def is_device_new(mac):
    query = join(DEVICES, lower_mac(mac))
    return not exists(query)
    
def is_feature_new(mac, feature):
    query = join(DEVICES, lower_mac(mac), feature)
    return not exists(query)

def lower_mac(mac):
    return mac.lower().replace(':', '.')

def get_target_path(mac, feature):
    target_path = join(DEVICES, lower_mac(mac), feature)
    if is_device_new(mac): mkdir(join(DEVICES, lower_mac(mac)))
    if is_feature_new(mac, feature): mkdir(target_path)
    return target_path
    
def mark_interval():
    while True:
        input('Press enter to mark start of interval')
        interval_start = time() - _start_time
        print(interval_start)
        input('Press enter to mark end of interval')
        interval_end = time() - _start_time
        print(interval_end)
        _interval_bounds.append((interval_start, interval_end))
    
def receive_packet(packet):
    _packet_timestamps.append(time() - _start_time)
    _packets.append(packet)

def get_training_set():
    training_set = []
    for index, packet in enumerate(_packets):
        timestamp = _packet_timestamps[index]
        feature_packet = False
        for interval_start, interval_end in _interval_bounds:
            if timestamp > interval_start and timestamp < interval_end:
                feature_packet = True
        tuple = (timestamp, packet, feature_packet)
        training_set.append(tuple)
    return training_set
    
def main():
    argument_parser = ArgumentParser()
    argument_parser.add_argument('--mac', required=True)
    argument_parser.add_argument('--feature', required=True)
    arguments = argument_parser.parse_args()
    
    t = Thread(target=mark_interval)
    t.setDaemon(True)
    t.start()
    sniff(filter='ether src host %s' % arguments.mac, iface='wlan0', timeout=180, prn=receive_packet)
    
    set = get_training_set()
    pickle_path = join(get_target_path(arguments.mac, arguments.feature), 'training.set')
    dump(set, open(pickle_path, 'wb'))
    print()
    
if __name__ == '__main__':
    main()
