from pathlib import Path
from pickle import load
from netfilterqueue import NetfilterQueue
from socket import fromfd, AF_UNIX, SOCK_STREAM
from scapy.all import *
from binascii import b2a_hex
from getmac import get_mac_address

_rf_models = {}
#_mac_addresses = set()

def get_training_model_paths():
    paths = []
    for path in Path('devices').rglob('training.model'):
        paths.append(str(path))
    return paths
        
def get_models():
    for training_model_path in get_training_model_paths():
        model, model_accuracy, mac_address, feature = load(open(training_model_path, 'rb'))
        if mac_address not in _rf_models:
            _rf_models[mac_address] = {}
        _rf_models[mac_address][feature] = (model, model_accuracy)

# def get_src_mac(nfq_packet):
#     mac_string = b2a_hex(nfq_packet.get_hw()).decode('ascii')[0:12].upper()
#     return ':'.join(mac_string[i:i+2] for i in range (0, len(mac_string), 2))

def get_mac(ip_address):
    mac = get_mac_address(ip=ip_address)
    if mac == None: return ''
    else: return mac
    
def get_secured_mac(src_mac, dst_mac):
    if src_mac in _rf_models: return src_mac
    if dst_mac in _rf_models: return dst_mac
    return None
    
def handle_nfq_packet(nfq_packet):
    scapy_packet = IP(nfq_packet.get_payload())
    size = len(scapy_packet) + 16 # (for missing ethernet frame)
    src_mac = get_mac(scapy_packet.src).upper()
    dst_mac = get_mac(scapy_packet.dst).upper()
    secured_mac = get_secured_mac(src_mac, dst_mac)
    print(scapy_packet.src + ' --> ' + scapy_packet.dst)
    print(src_mac + ' --> ' + dst_mac)
    print(secured_mac)
    print()
    nfq_packet.accept()
    return
    if secured_mac == None:
        print('Accepting\n')
        nfq_packet.accept()
        return
    print('no')
    ingress = int(secured_mac == dst_mac)
    for feature in _rf_models[secured_mac]:
        model, accuracy = _rf_models[secured_mac][feature]
        likely_feature_packet = make_prediction(model, [size, ingress])
        if likely_feature_packet:
            print('Dropping %s packet from %s' % (feature, secured_mac))
            nfq_packet.drop()
            return
    nfq_packet.accept()
    
def make_prediction(model, list):
    return model.predict([list]) # 0 benign/false, 1 feature/true
    
def main():
    get_models()
    #print(_rf_models)
    #_mac_addresses = set([tuple[2] for tuple in _rf_models])
    
    
    nfqueue = NetfilterQueue()
    nfqueue.bind(1, handle_nfq_packet)
    s = fromfd(nfqueue.get_fd(), AF_UNIX, SOCK_STREAM)
    
    try:
        print('Filtering has started')
        nfqueue.run_socket(s)
    except KeyboardInterrupt:
        pass
    
    s.close()
    nfqueue.unbind()
    
        
main()
