from json import dump, load
from sys import argv
from os import listdir
from os.path import isfile, join
from glob import glob

FINGERBANK_DIR = argv[1]
NMAP_DIR = argv[2]
P0F_DIR = argv[3]

def list_json(dir):
    return [f for f in listdir(dir) if isfile(join(dir, f)) and f.split('.')[-1] == 'json']

def fingerbank_set():
    return set([name.split('.fingerbank')[0] for name in list_json(FINGERBANK_DIR)])
    
def nmap_set():
    return set([name.split('.nmap')[0] for name in list_json(NMAP_DIR)])

def p0f_set():
    return set([name.split('.p0f')[0] for name in list_json(P0F_DIR)])

def set_union():
    return fingerbank_set() | nmap_set() | p0f_set()

def map_jsons(mac_address):
    mapping = {}
    for directory in [FINGERBANK_DIR, NMAP_DIR, P0F_DIR]:
        json_paths = glob("%s/%s*.json" % (directory, mac_address))
        for json_path in json_paths:
            mapping[directory.split('_')[0]] = load(open(json_path))
    return mapping
        
def main():
    for mac_address in set_union():
        print("Synthesizing fingerprint data for %s" % mac_address)
        merge = map_jsons(mac_address)
        with open(join('synthesized_json', "%s.fingerprint.json" % mac_address), 'w') as target:
            dump(merge, target)

main()
